#define vl_endianfun		/* define message structures */
#include "dns.api.h"
#undef vl_endianfun

/* instantiate all the print functions we know about */
#define vl_print(handle, ...) vlib_cli_output (handle, __VA_ARGS__)
#define vl_printfun
#include "dns.api.h"
#undef vl_printfun

#ifndef VL_API_DNS_ENABLE_DISABLE_REPLY_T_HANDLER
static void
vl_api_dns_enable_disable_reply_t_handler (vl_api_dns_enable_disable_reply_t * mp) {
   vat_main_t * vam = dns_test_main.vat_main;
   i32 retval = ntohl(mp->retval);
   if (vam->async_mode) {
      vam->async_errors += (retval < 0);
   } else {
      vam->retval = retval;
      vam->result_ready = 1;
   }
}
#endif
#ifndef VL_API_DNS_NAME_SERVER_ADD_DEL_REPLY_T_HANDLER
static void
vl_api_dns_name_server_add_del_reply_t_handler (vl_api_dns_name_server_add_del_reply_t * mp) {
   vat_main_t * vam = dns_test_main.vat_main;
   i32 retval = ntohl(mp->retval);
   if (vam->async_mode) {
      vam->async_errors += (retval < 0);
   } else {
      vam->retval = retval;
      vam->result_ready = 1;
   }
}
#endif
/* Only autoreply is supported (vl_api_dns_resolve_name_reply_t_handler()) */
/* Only autoreply is supported (vl_api_dns_resolve_ip_reply_t_handler()) */
static void
setup_message_id_table (vat_main_t * vam, u16 msg_id_base) {
   vl_msg_api_set_handlers(VL_API_DNS_ENABLE_DISABLE_REPLY + msg_id_base, "dns_enable_disable_reply",
                           vl_api_dns_enable_disable_reply_t_handler, vl_noop_handler,
                           vl_api_dns_enable_disable_reply_t_endian, vl_api_dns_enable_disable_reply_t_print,
                           sizeof(vl_api_dns_enable_disable_reply_t), 1);
   hash_set_mem (vam->function_by_name, "dns_enable_disable", api_dns_enable_disable);
   hash_set_mem (vam->help_by_name, "dns_enable_disable", "[enable][disable]");
   vl_msg_api_set_handlers(VL_API_DNS_NAME_SERVER_ADD_DEL_REPLY + msg_id_base, "dns_name_server_add_del_reply",
                           vl_api_dns_name_server_add_del_reply_t_handler, vl_noop_handler,
                           vl_api_dns_name_server_add_del_reply_t_endian, vl_api_dns_name_server_add_del_reply_t_print,
                           sizeof(vl_api_dns_name_server_add_del_reply_t), 1);
   hash_set_mem (vam->function_by_name, "dns_name_server_add_del", api_dns_name_server_add_del);
   hash_set_mem (vam->help_by_name, "dns_name_server_add_del", "<ip-address> [del]");
   vl_msg_api_set_handlers(VL_API_DNS_RESOLVE_NAME_REPLY + msg_id_base, "dns_resolve_name_reply",
                           vl_api_dns_resolve_name_reply_t_handler, vl_noop_handler,
                           vl_api_dns_resolve_name_reply_t_endian, vl_api_dns_resolve_name_reply_t_print,
                           sizeof(vl_api_dns_resolve_name_reply_t), 1);
   hash_set_mem (vam->function_by_name, "dns_resolve_name", api_dns_resolve_name);
   hash_set_mem (vam->help_by_name, "dns_resolve_name", "<hostname>");
   vl_msg_api_set_handlers(VL_API_DNS_RESOLVE_IP_REPLY + msg_id_base, "dns_resolve_ip_reply",
                           vl_api_dns_resolve_ip_reply_t_handler, vl_noop_handler,
                           vl_api_dns_resolve_ip_reply_t_endian, vl_api_dns_resolve_ip_reply_t_print,
                           sizeof(vl_api_dns_resolve_ip_reply_t), 1);
   hash_set_mem (vam->function_by_name, "dns_resolve_ip", api_dns_resolve_ip);
   hash_set_mem (vam->help_by_name, "dns_resolve_ip", "<ip4|ip6>");
}
clib_error_t * vat_plugin_register (vat_main_t *vam)
{
   dns_test_main_t * mainp = &dns_test_main;
   mainp->vat_main = vam;
   mainp->msg_id_base = vl_client_get_first_plugin_msg_id ("dns_269575cd");
   if (mainp->msg_id_base == (u16) ~0)
      return clib_error_return (0, "dns plugin not loaded...");
   setup_message_id_table (vam, mainp->msg_id_base);
#ifdef VL_API_LOCAL_SETUP_MESSAGE_ID_TABLE
    VL_API_LOCAL_SETUP_MESSAGE_ID_TABLE(vam);
#endif
   return 0;
}
