#define vl_endianfun		/* define message structures */
#include "rdma.api.h"
#undef vl_endianfun

/* instantiate all the print functions we know about */
#define vl_print(handle, ...) vlib_cli_output (handle, __VA_ARGS__)
#define vl_printfun
#include "rdma.api.h"
#undef vl_printfun

/* Only autoreply is supported (vl_api_rdma_create_reply_t_handler()) */
#ifndef VL_API_RDMA_DELETE_REPLY_T_HANDLER
static void
vl_api_rdma_delete_reply_t_handler (vl_api_rdma_delete_reply_t * mp) {
   vat_main_t * vam = rdma_test_main.vat_main;
   i32 retval = ntohl(mp->retval);
   if (vam->async_mode) {
      vam->async_errors += (retval < 0);
   } else {
      vam->retval = retval;
      vam->result_ready = 1;
   }
}
#endif
static void
setup_message_id_table (vat_main_t * vam, u16 msg_id_base) {
   vl_msg_api_set_handlers(VL_API_RDMA_CREATE_REPLY + msg_id_base, "rdma_create_reply",
                           vl_api_rdma_create_reply_t_handler, vl_noop_handler,
                           vl_api_rdma_create_reply_t_endian, vl_api_rdma_create_reply_t_print,
                           sizeof(vl_api_rdma_create_reply_t), 1);
   hash_set_mem (vam->function_by_name, "rdma_create", api_rdma_create);
   hash_set_mem (vam->help_by_name, "rdma_create", "<host-if ifname> [name <name>] [rx-queue-size <size>] [tx-queue-size <size>] [num-rx-queues <size>] [mode <auto|ibv|dv>]");
   vl_msg_api_set_handlers(VL_API_RDMA_DELETE_REPLY + msg_id_base, "rdma_delete_reply",
                           vl_api_rdma_delete_reply_t_handler, vl_noop_handler,
                           vl_api_rdma_delete_reply_t_endian, vl_api_rdma_delete_reply_t_print,
                           sizeof(vl_api_rdma_delete_reply_t), 1);
   hash_set_mem (vam->function_by_name, "rdma_delete", api_rdma_delete);
   hash_set_mem (vam->help_by_name, "rdma_delete", "<sw_if_index index>");
}
clib_error_t * vat_plugin_register (vat_main_t *vam)
{
   rdma_test_main_t * mainp = &rdma_test_main;
   mainp->vat_main = vam;
   mainp->msg_id_base = vl_client_get_first_plugin_msg_id ("rdma_e22a9d05");
   if (mainp->msg_id_base == (u16) ~0)
      return clib_error_return (0, "rdma plugin not loaded...");
   setup_message_id_table (vam, mainp->msg_id_base);
#ifdef VL_API_LOCAL_SETUP_MESSAGE_ID_TABLE
    VL_API_LOCAL_SETUP_MESSAGE_ID_TABLE(vam);
#endif
   return 0;
}
